# Answer

gendata <- function(n, m, stdev, distype = 1) {
	result <- NULL
	if(distype == 1) {
		result <- rnorm(n, 0, 1)
	} else if (distype == 2) {
		result <- rt(n, 3)
	} else if (distype == 3) {
		result <- rchisq(n, 4)
	} else {
		stop("The distype argument is not valid.")
	}
	result <- (result - mean(result)) / sd(result)
	
	# Find empirical mean and sd
	temp <- rnorm(n, m, stdev)
	result <- sd(temp) * result + mean(temp)
	return(result)
}

runsim <- function(cond) {
	b0 <- 0
	b1 <- unlist(cond[1])
	n <- unlist(cond[2])
	distype <- unlist(cond[3])
	x <- gendata(n, 0, 1, distype = distype)
	fy <- b0 + b1*x
	p <- exp(fy)/(1 + exp(fy))
	y <- rep(NA, n)
	for(w in 1:n) y[w] <- rbinom(1, 1, p[w])
	dat <- data.frame(x = x, y = y)
	out <- glm(y ~ x, data = dat, family = binomial(link = "logit"))
	sumout <- summary(out)
	sumout[["coefficients"]][2, c(1, 2, 4)]
}

runrep <- function(repseed) {
	RNGkind("L'Ecuyer-CMRG")
		
	# Assign the seed for each rep inside each node
	assign(".Random.seed", repseed, envir = .GlobalEnv)
	bs <- c(0, 0.5, 1, 1.5)
	ns <- c(50, 100, 200, 400)
	distypes <- c(1, 2, 3)
	conds <- expand.grid(bs, ns, distypes)
	dfconds <- data.frame(t(conds))
	result <- lapply(dfconds, runsim)
	result <- do.call(rbind, result)
	result <- cbind(conds, result)
	colnames(result) <- c("b", "n", "distype", "est", "se", "p")
	result
}

# Parallel processing with L'Ecuyer seed
set.seed(123321)
nrep <- 1000
library(parallel)
RNGkind("L'Ecuyer-CMRG") # Use L'Ecuyer method
seed.l <- list() # Create list of seed number to provide for each rep
s <- .Random.seed # Extract the current seed number
for (i in 1:nrep) {
  seed.l[[i]] <- s # Save seed number to the list
  s <- nextRNGStream(s) # Build independent seed number
}

# Use mclapply to distribute seed number for each rep
result <- mclapply(seed.l, runrep, mc.cores = 11)
result <- do.call(rbind, result)
save(result, file=paste("key7ex7.Rdata", sep=""))
